@@ -48,6 +48,10 @@ The following source types are supported for bucket deployments:
 * Local .zip file: `s3deploy.Source.asset('/path/to/local/file.zip')`
 * Local directory: `s3deploy.Source.asset('/path/to/local/directory')`
 * Another bucket: `s3deploy.Source.bucket(bucket, zipObjectKey)`
+* String data: `s3deploy.Source.data('object-key.txt', 'hello, world!')`
+  (supports [deploy-time values](#data-with-deploy-time-values))
+* JSON data: `s3deploy.Source.jsonData('object-key.json', { json: 'object' })`
+  (supports [deploy-time values](#data-with-deploy-time-values))
 
 To create a source from a single file, you can pass `AssetOptions` to exclude
 all but a single file:
@@ -272,6 +276,35 @@ s3deploy.BucketDeployment(self, "DeployMeWithEfsStorage",
 )
 ```
 
+## Data with deploy-time values
+
+The content passed to `Source.data()` or `Source.jsonData()` can include
+references that will get resolved only during deployment.
+
+For example:
+
+```python
+import aws_cdk.aws_sns as sns
+
+# destination_bucket: s3.Bucket
+# topic: sns.Topic
+
+
+app_config = {
+    "topic_arn": topic.topic_arn,
+    "base_url": "https://my-endpoint"
+}
+
+s3deploy.BucketDeployment(self, "BucketDeployment",
+    sources=[s3deploy.Source.json_data("config.json", app_config)],
+    destination_bucket=destination_bucket
+)
+```
+
+The value in `topic.topicArn` is a deploy-time value. It only gets resolved
+during deployment by placing a marker in the generated source file and
+substituting it when its deployed to the destination with the actual value.
+
 ## Notes
 
 * This library uses an AWS CloudFormation custom resource which about 10MiB in
@@ -338,15 +371,17 @@ class BucketDeployment(
 
     Example::
 
-        website_bucket = s3.Bucket(self, "WebsiteBucket",
-            website_index_document="index.html",
-            public_read_access=True
-        )
+        # destination_bucket: s3.Bucket
+        # vpc: ec2.Vpc
         
-        s3deploy.BucketDeployment(self, "DeployWebsite",
-            sources=[s3deploy.Source.asset("./website-dist")],
-            destination_bucket=website_bucket,
-            destination_key_prefix="web/static"
+        
+        s3deploy.BucketDeployment(self, "DeployMeWithEfsStorage",
+            sources=[s3deploy.Source.asset(path.join(__dirname, "my-website"))],
+            destination_bucket=destination_bucket,
+            destination_key_prefix="efs/",
+            use_efs=True,
+            vpc=vpc,
+            retain_on_delete=False
         )
     '''
 
@@ -552,15 +587,17 @@ class BucketDeploymentProps:
 
         Example::
 
-            website_bucket = s3.Bucket(self, "WebsiteBucket",
-                website_index_document="index.html",
-                public_read_access=True
-            )
+            # destination_bucket: s3.Bucket
+            # vpc: ec2.Vpc
             
-            s3deploy.BucketDeployment(self, "DeployWebsite",
-                sources=[s3deploy.Source.asset("./website-dist")],
-                destination_bucket=website_bucket,
-                destination_key_prefix="web/static"
+            
+            s3deploy.BucketDeployment(self, "DeployMeWithEfsStorage",
+                sources=[s3deploy.Source.asset(path.join(__dirname, "my-website"))],
+                destination_bucket=destination_bucket,
+                destination_key_prefix="efs/",
+                use_efs=True,
+                vpc=vpc,
+                retain_on_delete=False
             )
         '''
         if isinstance(metadata, dict):
@@ -1262,20 +1299,24 @@ class Source(metaclass=jsii.JSIIMeta, jsii_type="@aws-cdk/aws-s3-deployment.Sour
         Source.bucket(bucket, key)
         Source.asset('/local/path/to/directory')
         Source.asset('/local/path/to/a/file.zip')
+        Source.data('hello/world/file.txt', 'Hello, world!')
+        Source.data('config.json', { baz: topic.topicArn })
 
     :exampleMetadata: infused
 
     Example::
 
-        website_bucket = s3.Bucket(self, "WebsiteBucket",
-            website_index_document="index.html",
-            public_read_access=True
-        )
+        # destination_bucket: s3.Bucket
+        # vpc: ec2.Vpc
         
-        s3deploy.BucketDeployment(self, "DeployWebsite",
-            sources=[s3deploy.Source.asset("./website-dist")],
-            destination_bucket=website_bucket,
-            destination_key_prefix="web/static"
+        
+        s3deploy.BucketDeployment(self, "DeployMeWithEfsStorage",
+            sources=[s3deploy.Source.asset(path.join(__dirname, "my-website"))],
+            destination_bucket=destination_bucket,
+            destination_key_prefix="efs/",
+            use_efs=True,
+            vpc=vpc,
+            retain_on_delete=False
         )
     '''
 
@@ -1341,11 +1382,45 @@ class Source(metaclass=jsii.JSIIMeta, jsii_type="@aws-cdk/aws-s3-deployment.Sour
         '''
         return typing.cast(ISource, jsii.sinvoke(cls, "bucket", [bucket, zip_object_key]))
 
+    @jsii.member(jsii_name="data") # type: ignore[misc]
+    @builtins.classmethod
+    def data(cls, object_key: builtins.str, data: builtins.str) -> ISource:
+        '''Deploys an object with the specified string contents into the bucket.
+
+        The
+        content can include deploy-time values (such as ``snsTopic.topicArn``) that
+        will get resolved only during deployment.
+
+        To store a JSON object use ``Source.jsonData()``.
+
+        :param object_key: The destination S3 object key (relative to the root of the S3 deployment).
+        :param data: The data to be stored in the object.
+        '''
+        return typing.cast(ISource, jsii.sinvoke(cls, "data", [object_key, data]))
+
+    @jsii.member(jsii_name="jsonData") # type: ignore[misc]
+    @builtins.classmethod
+    def json_data(cls, object_key: builtins.str, obj: typing.Any) -> ISource:
+        '''Deploys an object with the specified JSON object into the bucket.
+
+        The
+        object can include deploy-time values (such as ``snsTopic.topicArn``) that
+        will get resolved only during deployment.
+
+        :param object_key: The destination S3 object key (relative to the root of the S3 deployment).
+        :param obj: A JSON object.
+        '''
+        return typing.cast(ISource, jsii.sinvoke(cls, "jsonData", [object_key, obj]))
+
 
 @jsii.data_type(
     jsii_type="@aws-cdk/aws-s3-deployment.SourceConfig",
     jsii_struct_bases=[],
-    name_mapping={"bucket": "bucket", "zip_object_key": "zipObjectKey"},
+    name_mapping={
+        "bucket": "bucket",
+        "zip_object_key": "zipObjectKey",
+        "markers": "markers",
+    },
 )
 class SourceConfig:
     def __init__(
@@ -1353,11 +1428,13 @@ class SourceConfig:
         *,
         bucket: aws_cdk.aws_s3.IBucket,
         zip_object_key: builtins.str,
+        markers: typing.Optional[typing.Mapping[builtins.str, typing.Any]] = None,
     ) -> None:
         '''Source information.
 
         :param bucket: The source bucket to deploy from.
         :param zip_object_key: An S3 object key in the source bucket that points to a zip file.
+        :param markers: A set of markers to substitute in the source content. Default: - no markers
 
         :exampleMetadata: fixture=_generated
 
@@ -1369,16 +1446,24 @@ class SourceConfig:
             import aws_cdk.aws_s3_deployment as s3_deployment
             
             # bucket: s3.Bucket
+            # markers: Any
             
             source_config = s3_deployment.SourceConfig(
                 bucket=bucket,
-                zip_object_key="zipObjectKey"
+                zip_object_key="zipObjectKey",
+            
+                # the properties below are optional
+                markers={
+                    "markers_key": markers
+                }
             )
         '''
         self._values: typing.Dict[str, typing.Any] = {
             "bucket": bucket,
             "zip_object_key": zip_object_key,
         }
+        if markers is not None:
+            self._values["markers"] = markers
 
     @builtins.property
     def bucket(self) -> aws_cdk.aws_s3.IBucket:
@@ -1394,6 +1479,15 @@ class SourceConfig:
         assert result is not None, "Required property 'zip_object_key' is missing"
         return typing.cast(builtins.str, result)
 
+    @builtins.property
+    def markers(self) -> typing.Optional[typing.Mapping[builtins.str, typing.Any]]:
+        '''A set of markers to substitute in the source content.
+
+        :default: - no markers
+        '''
+        result = self._values.get("markers")
+        return typing.cast(typing.Optional[typing.Mapping[builtins.str, typing.Any]], result)
+
     def __eq__(self, rhs: typing.Any) -> builtins.bool:
         return isinstance(rhs, self.__class__) and rhs._values == self._values
 
